//
//  switchDiskSizeBase.m
//  switchDiskSizeBase v2.1
//
//  Copyright (c) 2012-2014, brkirch
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice, this
//  list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//  this list of conditions and the following disclaimer in the documentation
//  and/or other materials provided with the distribution.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include <dlfcn.h>
#include <stdint.h>
#include <stdlib.h>
#import <Foundation/Foundation.h>

static NSString * (*orig_NSLocalizedFileSizeDescription) (uint64_t arg1, uint64_t arg2, int flags) = NULL;

NSString * NSLocalizedFileSizeDescription(uint64_t arg1, uint64_t arg2, int flags)
{
#if defined(BINUNITS) || defined(DUALUNITS)
    NSUInteger numReplacements = 0;
#endif
    
    if (!orig_NSLocalizedFileSizeDescription)
        orig_NSLocalizedFileSizeDescription = dlsym(RTLD_NEXT, "NSLocalizedFileSizeDescription");
    
#if defined(BINUNITS) || defined(DUALUNITS)
    NSMutableString *sizeString = [NSMutableString stringWithString:orig_NSLocalizedFileSizeDescription(arg1, arg2, flags & 0xFFFBFFFF | 0x180000)];
    
    numReplacements += [sizeString replaceOccurrencesOfString:@" YB" withString:@" YiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" ZB" withString:@" ZiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" EB" withString:@" EiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" PB" withString:@" PiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" TB" withString:@" TiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" GB" withString:@" GiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" MB" withString:@" MiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
    numReplacements += [sizeString replaceOccurrencesOfString:@" KB" withString:@" KiB" options:NSLiteralSearch range:NSMakeRange(0, [sizeString length])];
#endif
    
#ifdef DUALUNITS
    NSString *base10SizeString = orig_NSLocalizedFileSizeDescription(arg1, arg2, flags & 0xFFE7FFFF);
    if (numReplacements != 0 && ![sizeString isEqual:base10SizeString])
    {
        [sizeString appendString:@" / "];
        [sizeString appendString:base10SizeString];
    }
#endif
    
#if defined(BINUNITS) || defined(DUALUNITS)
    return sizeString;
#else
    return orig_NSLocalizedFileSizeDescription(arg1, arg2, flags & 0xFFFBFFFF | 0x180000);
#endif
}
